#!/usr/bin/perl

use Cwd;
use File::Find;
use File::Copy;
use File::Path qw(make_path);

use strict;
package main;

($main::program = $0) =~ s,.*[\\/],,;
$main::program =~ s|\.pl||;
$main::program =~ s|\.exe||;

$| = 1;  # turn off buffering of STDOUT output

initialize ();

my $inputDir = "$main::ini{'GnuPlotDirectory'}/$main::ini{'ProjectDirectory'}";
$inputDir =~ s|\.\.\.|\.\.|;
$main::ini{'InputDirectory'} = $inputDir;

find_files ();
filter_files ();
create_report_files ();
run ();
close_report_files ();
terminate ();

#-------------------------------------------------------
sub run
{
   print <<_END;

============================================================
Plot individual plates
============================================================

_END

   read_script_template ();
   process_files ();
   write_report ();
}

#-------------------------------------------------------
sub create_report_files
{
   my $path = $main::ini{'HtmlHeaderFile'};
   open (IN, $path) or error_exit ("could not open [$path]!\n");
   @main::htmlHeader = <IN>;
   close (IN);

   $path = $main::ini{'HtmlFooterFile'};
   open (IN, $path) or error_exit ("could not open [$path]!\n");
   @main::htmlFooter = <IN>;
   close (IN);

   my $status = mkdir ($main::ini{'ReportDirectory'});

   $path = "$main::ini{'ReportDirectory'}/$main::ini{'ReportFile'}.txt";
   open (TXT, ">$path") or error_exit ("could not open [$path] for writing!\n");
   
   $path = "$main::ini{'ReportDirectory'}/$main::ini{'ReportFile'}.csv";
   open (CSV, ">$path") or error_exit ("could not open [$path] for writing!\n");
   
   $path = "$main::ini{'ReportDirectory'}/$main::ini{'ReportFile'}.html";
   open (HTM, ">$path") or error_exit ("could not open [$path] for writing!\n");
   
   print HTM @main::htmlHeader; 
}

#-------------------------------------------------------
sub close_report_files
{
   print HTM @main::htmlFooter; 

   close (TXT);
   close (CSV);
   close (HTM);

   my $path = "$main::ini{'ReportDirectory'}/$main::ini{'ReportFile'}.html";
   my $command = "start $path";
   system ($command);
}

#-------------------------------------------------------
sub filter_files
{
   my @files;
   foreach (@main::files) {
      my $path = $_;
      if ($path =~ m|$main::ini{'SheetFileName'}\.$main::ini{'FileExtension'}|) {
         push @files, $path;
      }
   }
   @main::files = @files;
}

#-------------------------------------------------------
sub rewrite_data_sheets
{
   foreach (@main::files) {
      my $path = $_;
      my $pathSave = $path;
      $path =~ s|$main::ini{'InputDirectory'}/||;

      my @fields = split /\//, $path;     
      if ($fields[-1] eq "sheet.txt") { pop @fields };
      if ($fields[-1] eq "data") { pop @fields };
      my ($replicate) = pop @fields;
      my ($compound) = pop @fields;
      
      my $record = $main::recordAcceptCompound{$compound};
      open (TXT, $pathSave) or error_exit ("could not open [$pathSave]\n");
      my @txt = <TXT>;
      close (TXT);
      if ($txt[0] =~ m|INCLUDE|) {
         shift @txt;
      }
      my @txtOut;
      push @txtOut, "INCLUDE\t$record\n";
      push @txtOut, @txt;
      print "rewriting [$pathSave]...\n";
      open (TXT, ">$pathSave") or error_exit ("could not open [$pathSave] for writing!\n");
      print TXT @txtOut;
   }   
}

#-------------------------------------------------------
sub analyze_acceptance
{
   my @acceptColumn;
   foreach (sort keys %main::recordAccept) {
      my $key = $_;
      my @fields = split /\//, $key;
      my ($replicate) = pop @fields;
      my ($compound) = pop @fields;
      $key = "$compound/$replicate";
      $replicate =~ s|R||;
      if ($replicate == 1) {
         undef @acceptColumn;
         my $i = 0;
         for (; $i < $main::ini{'Columns'}; $i++) {
            push @acceptColumn, "+";
         }
      }
      my @record = split /\t/, $main::recordAccept{$key};
      my $i = 0;
      for (; $i < $main::ini{'Columns'}; $i++) {
         if ($record[$i] ne "+") {
            $acceptColumn[$i] = "-";
         }
      }
      if ($replicate == 3) {
         my $record = join "\t", @acceptColumn;
         $main::recordAcceptCompound{$compound} = $record;
      }
   }
}

#-------------------------------------------------------
sub write_report
{
   write_report_txt ();
   write_report_csv ();
   write_report_html ();
}

#-------------------------------------------------------
sub write_report_txt
{
}

#-------------------------------------------------------
sub write_report_csv
{
}

#-------------------------------------------------------
sub write_report_html
{
   print HTM $main::htmlReport;
}

#-------------------------------------------------------
sub create_link
{
   my ($key) = @_;
   my ($compound, $replicate) = split /\//, $key;
   my $prefix = "../";  # we have to back-track from the "results" directory!   This should be made programmatic.
   
   my $link = "<a href=\"$prefix$main::ini{'InputDirectory'}/$compound/$replicate/output/$main::ini{'ScriptFile'}/index.html\">";
   $link =~ s|\.txt||;
   $link .= "$key</a>";
   return $link;
}

#-------------------------------------------------------
sub read_script_template
{
   my $path = $main::ini{'ScriptTemplate'};
   open (IN, $path) or error_exit ("could not open [$path]!\n");
   @main::scriptTemplate = <IN>;
   close (IN);
}

#-------------------------------------------------------
sub process_files
{
   foreach (@main::files) {
      my $path = $_;
      process_script ($path);
   }
}

#-------------------------------------------------------
sub process_script
{
   my ($path) = @_;
   read_data ($path);
   my $script = write_script ($path);
   execute_script ($script);
   parse_output ($script);
}

#-------------------------------------------------------
sub read_data
{
   my ($path) = @_;

   open (IN, $path) or error_exit ("could not open [$path]!\n");
   my @txt = <IN>;
   close (IN);
      
   foreach (@txt) {
      if (/^\[I\]/) {
         chomp ();
         my @fields = split /\t/;         
         my $header = shift @fields;
         my ($label, $unit) = split ",", $header;
         my $multInput = 1;
         if ($unit =~ m|pM|) {
            $multInput = 1.e-12;
         }
         elsif ($unit =~ m|nM|) {
            $multInput = 1.e-9;
         }
         elsif ($unit =~ m|uM|) {
            $multInput = 1.e-6;
         }
         elsif ($unit =~ m|mM|) {
            $multInput = 1.e-3;
         }
         elsif ($unit =~ m|M|) {
            $multInput = 1;
         }
         else {
            print "ERROR: wrong unit for inhibitor concentration.\n",
                  "Allowable units are M, mM, uM, nM, and pM.";
            termninate ();
         }
         my $multConc = conc_scale_factor ();
         my $mult = $multInput/$multConc;        
         my $conc;
         foreach $conc (@fields) {
            $conc *= $mult;
            $conc = sprintf ("%.2f", $conc);
         }
         @main::concs = @fields;
      }
   }
}

#-------------------------------------------------------
sub conc_scale_factor 
{
   my $mult = 1;
   my $unit = $main::ini{'ConcentrationUnit'};
   if ($unit =~ m|pM|) {
      $mult = 1.e-12;
   }
   elsif ($unit =~ m|nM|) {
      $mult = 1.e-9;
   }
   elsif ($unit =~ m|uM|) {
      $mult = 1.e-6;
   }
   elsif ($unit =~ m|mM|) {
      $mult = 1.e-3;
   }
   elsif ($unit =~ m|M|) {
      $mult = 1;
   }
   else {
      print "ERROR: wrong concentration unit.\n",
            "Allowable units are M, mM, uM, nM, and pM.";
      termninate ();
   }
   return $mult;
}

#-------------------------------------------------------
sub parse_output
{
   my ($script) = @_;
}
 
#-------------------------------------------------------
sub execute_script
{   
   my ($script) = @_;
   my $cwd = cwd ();
   $script =~ s|$main::ini{'InputDirectory'}|$main::ini{'ProjectDirectory'}|;
   my $log = $script;
   $log =~ s|\.txt$|.log|;

   chdir ($main::ini{'GnuPlotDirectory'});
   my $command = "$main::ini{'GnuPlotExe'} $script";
   my $screen = $script;
   $screen =~ s|$main::ini{'ProjectDirectory'}/||;
   $screen =~ s|/$main::ini{'ScriptFile'}||;
   $screen =~ s|/| : |;
   print $screen, " ... ";
   my $status = system ($command);
   print " status = $status\n";
   chdir ($cwd)
}

#-------------------------------------------------------
sub write_script
{
   my ($pathData) = @_;
   
   my $pathScript = $pathData;
   my $pathSplit = $pathData;

   my @fields = split /\//, $pathSplit;     
   if ($fields[-1] eq "sheet.txt") { pop @fields };
   if ($fields[-1] eq "data") { pop @fields };
   my ($replicate) = pop @fields;
   my ($compound) = pop @fields;
      
   $pathScript =~ s|/[^/]+$||;
   $pathScript =~ s|/data||;

   my $outputDir = $pathScript;
   $outputDir =~ s|$main::ini{'InputDirectory'}|$main::ini{'ProjectDirectory'}|;
   $outputDir .= "/output/$main::ini{'ScriptFile'}";
   $outputDir =~ s|\.txt$||;
   
   my $dataDir = $pathScript;
   $dataDir =~ s|$main::ini{'InputDirectory'}|$main::ini{'ProjectDirectory'}|;
   $dataDir .= "/data";

   $pathScript .= "/$main::ini{'ScriptFile'}";

   my @script = @main::scriptTemplate;
   my $pathSheet = "$dataDir/$main::ini{'SheetFileName'}.$main::ini{'FileExtension'}";
   my $title = "$compound | $replicate";
   my $line;
   foreach $line (@script) {      
      $line =~ s|TITLE|$title|;
      $line =~ s|OUTPUTDIR|$outputDir|;
      $line =~ s|SHEET|$pathSheet|;
      $line =~ s|A01|$main::concs[0] $main::ini{'ConcentrationUnit'}|;
      $line =~ s|A02|$main::concs[1] $main::ini{'ConcentrationUnit'}|;
      $line =~ s|A03|$main::concs[2] $main::ini{'ConcentrationUnit'}|;
      $line =~ s|A04|$main::concs[3] $main::ini{'ConcentrationUnit'}|;
      $line =~ s|A05|$main::concs[4] $main::ini{'ConcentrationUnit'}|;
      $line =~ s|A06|$main::concs[5] $main::ini{'ConcentrationUnit'}|;
      $line =~ s|A07|$main::concs[6] $main::ini{'ConcentrationUnit'}|;
      $line =~ s|A08|$main::concs[7] $main::ini{'ConcentrationUnit'}|;
      $line =~ s|A09|$main::concs[8] $main::ini{'ConcentrationUnit'}|;
      $line =~ s|A10|$main::concs[9] $main::ini{'ConcentrationUnit'}|;
      $line =~ s|A11|$main::concs[10] $main::ini{'ConcentrationUnit'}|;
      $line =~ s|A12|$main::concs[11] $main::ini{'ConcentrationUnit'}|;
   }
   
   my $pathImg = $outputDir;
   $pathImg =~ s|$main::ini{'ProjectDirectory'}|$main::ini{'InputDirectory'}|;
   $pathImg = "../" . $pathImg;
   if ($replicate eq "R1") {
      $main::htmlReport .= "\n<h2>$compound</h2>\n";
   }
   $main::htmlReport .= "\n<h4>$compound : Replicate $replicate</h4>\n";
   $main::htmlReport .= "<img src=\"$pathImg/sheet-001.gif\">\n";
   $main::htmlReport .= "<img src=\"$pathImg/sheet-002.gif\"><br>\n";

   open (OUT, ">$pathScript") or die "could not open [$pathScript]!\n";
   print OUT @script;
   close (OUT);
   
   my $cwd = cwd ();
   chdir ($main::ini{'GnuPlotDirectory'});
   make_path ($outputDir);
   chdir ($cwd);

   return $pathScript;
}

#-------------------------------------------------------
#-------------------------------------------------------
sub initialize
{
	my ($line, $inifile, $logfile);

	if (@ARGV) {
		$inifile = $ARGV[0];
	}
	else {
		$inifile = "$main::program.ini";
	}
	read_inifile ($inifile);

	$logfile = "$main::program.log";
	open (LOG, ">$logfile") or die $!
}

#-------------------------------------------------------
sub read_inifile
{
	my ($inifile) = @_;
	my ($line, $lineRaw, $key, $value, $contin);

	open (INI, $inifile) or die "could not open initialization file [$inifile]!\n";

	$contin = 0;
	$line = "";

	lineRaw:
	foreach $lineRaw (<INI>) {
		chomp ($lineRaw);
		
		# Allow for continuation lines:

		if ($lineRaw =~ m|\\$|) {
			$contin = 1;
		   $lineRaw =~ s|\\$||;
			$line .= $lineRaw;
			next lineRaw;
		}
		else {
			if ($contin) {
				$line .= $lineRaw;
			}
			else {
				$line = $lineRaw;
			}
			$contin = 0;
		}

		$line =~ s|\;.+||g;
		if (!($line =~ m|=|)) {
			next;
		}

		($key, $value) = split /=/, $line;
		$key =~ s|^\s+||;
		$key =~ s|\s+$||;
		$value =~ s|^\s+||;
		$value =~ s|\s+$||;
		$main::ini{$key} = $value;

		$line = "";
	}
	close (INI);
}

#-------------------------------------------------
sub find_files_dir_ext
{
	my ($directory, $extension) = @_;

	undef @main::files;

	$main::match = "";
	if (length($extension) > 0) {
		$main::match = "\.$extension";
	}
	find (\&wanted_files, $directory);

	return @main::files;
}

#-------------------------------------------------
sub find_all_files
{
	my (@tmp);

	@tmp = find_file_extension ("$main::ini{'InputDirectory'}", "", "");

	return @tmp;
}

#-------------------------------------------------
sub find_files
{
	my (@tmp);

	@tmp = find_file_extension ("$main::ini{'InputDirectory'}", "", $main::ini{'FileExtension'});

	return @tmp;
}

#-------------------------------------------------
sub find_file_extension
{
	my ($directory, $file, $extension) = @_;

	undef @main::files;

	$main::match = "";
	if (defined($file) && length($file) > 0) {
		$main::match = $file;
	}
	elsif (length($extension) > 0) {
		$main::match = "\.$extension";
	}

	find (\&wanted_files, $directory);

	return @main::files;
}

#-------------------------------------------------
sub wanted_files
{
	m|$main::match$|i && push @main::files, $File::Find::name;
}

#-------------------------------------------------------
sub terminate
{
	close (LOG);
	if ((defined $main::ini{'PressEnterToFinish'}) && ($main::ini{'PressEnterToFinish'} eq 'n')) {
	   return;
	}
	print "All done.  Press ENTER to terminate.";
	<STDIN>;
}

#-------------------------------------------------------
sub error_exit
{
   my ($message) = @_;
   print "ERROR: $message\n\nPress [Enter] to terminate.";
   <STDIN>;
   exit ();   
}
