#!/usr/bin/perl

use Cwd;
use File::Find;
use File::Copy;
use File::Path;    		# $status = mkpath ($dir);
use File::Basename; 		# ($name,$path,$suffix) = fileparse ($fullname);

use strict;
package main;

($main::program = $0) =~ s,.*[\\/],,;
$main::program =~ s|\.pl||;
$main::program =~ s|\.exe||;

$| = 1;  # turn off buffering of STDOUT output

initialize ();
run ();
terminate ();

#-------------------------------------------------------
sub run
{
   my $cwd = cwd ();
   
   my $perl = "perl -w ";
   my $ext = ".pl";
   if ($main::ini{'IsPerlInstalled'} ne "y") {
      $perl = "";
      $ext = ".exe";
   }
   if ($main::ini{'PlotRawData'} eq "y") {
      chdir ("01-plot-raw-data");
      my $command = $perl . "plot-raw-data" . $ext;
      system ($command);
      chdir ($cwd)
   }  
   if ($main::ini{'ExcludeOutliers'} eq "y") {
      chdir ("02-exclude-outliers");
      my $command = $perl . "exclude-outliers" . $ext;
      system ($command);
      chdir ($cwd)
   }  
   if ($main::ini{'DetermineInitialRates'} eq "y") {
      chdir ("03-initial-rates");
      my $command = $perl . "initial-rates" . $ext;
      system ($command);
      chdir ($cwd)
   }  
   if ($main::ini{'DetermineKiApp'} eq "y") {
      chdir ("04-kiapp");
      my $command = $perl . "kiapp" . $ext;
      system ($command);
      chdir ($cwd)
   }  
   if ($main::ini{'DetermineKinactKi'} eq "y") {
      chdir ("05-ki-kinact");
      my $command = $perl . "ki-kinact" . $ext;
      system ($command);
      chdir ($cwd)
   }  
   if ($main::ini{'CorrelationWithCellIC50'} eq "y") {
      chdir ("06-ic50-correl");
      my $command = $perl . "ic50-correl" . $ext;
      system ($command);
      chdir ($cwd)
   }  
}

#-------------------------------------------------------
#-------------------------------------------------------
sub initialize
{
	my ($line, $inifile, $logfile);

	if (@ARGV) {
		$inifile = $ARGV[0];
	}
	else {
		$inifile = "$main::program.ini";
	}
	read_inifile ($inifile);

	$logfile = "$main::program.log";
	open (LOG, ">$logfile") or error_exit ("could not open log file\n[$logfile]\n");
}

#-------------------------------------------------------
sub read_inifile
{
	my ($inifile) = @_;
	my ($line, $lineRaw, $key, $value, $contin);

	open (INI, $inifile) or error_exit ("could not open initialization file\n[$inifile]\n");

	$contin = 0;
	$line = "";

	lineRaw:
	foreach $lineRaw (<INI>) {
		chomp ($lineRaw);
		
		# Allow for continuation lines:

		if ($lineRaw =~ m|\\$|) {
			$contin = 1;
		   $lineRaw =~ s|\\$||;
			$line .= $lineRaw;
			next lineRaw;
		}
		else {
			if ($contin) {
				$line .= $lineRaw;
			}
			else {
				$line = $lineRaw;
			}
			$contin = 0;
		}

		$line =~ s|\;.+||g;
		if (!($line =~ m|=|)) {
			next;
		}

		($key, $value) = split /=/, $line;
		$key =~ s|^\s+||;
		$key =~ s|\s+$||;
		$value =~ s|^\s+||;
		$value =~ s|\s+$||;
		$main::ini{$key} = $value;

		$line = "";
	}
	close (INI);
}

#-------------------------------------------------
sub find_files_dir_ext
{
	my ($directory, $extension) = @_;

	undef @main::files;

	$main::match = "";
	if (length($extension) > 0) {
		$main::match = "\.$extension";
	}
	find (\&wanted_files, $directory);

	return @main::files;
}

#-------------------------------------------------
sub find_all_files
{
	my (@tmp);

	@tmp = find_file_extension ("$main::ini{'InputDirectory'}", "", "");

	return @tmp;
}

#-------------------------------------------------
sub find_files
{
	my (@tmp);

	@tmp = find_file_extension ("$main::ini{'InputDirectory'}", "", $main::ini{'FileExtension'});

	return @tmp;
}

#-------------------------------------------------
sub find_file_extension
{
	my ($directory, $file, $extension) = @_;

	undef @main::files;

	$main::match = "";
	if (defined($file) && length($file) > 0) {
		$main::match = $file;
	}
	elsif (length($extension) > 0) {
		$main::match = "\.$extension";
	}

	find (\&wanted_files, $directory);

	return @main::files;
}

#-------------------------------------------------
sub wanted_files
{
	m|$main::match$|i && push @main::files, $File::Find::name;
}

#-------------------------------------------------------
sub terminate
{
	close (LOG);
	if ((defined $main::ini{'PressEnterToFinish'}) && ($main::ini{'PressEnterToFinish'} eq 'n')) {
	   return;
	}
	print "All done.  Press ENTER to terminate.";
	<STDIN>;
}

#-------------------------------------------------------
sub error_exit
{
   my ($message) = @_;
   print "\n\nERROR: $message\n\nPress [Enter] to terminate.";
   <STDIN>;
   exit ();   
}

